@@ -44,6 +44,7 @@ var LibraryEmbind = {
   },
   $FunctionDefinition__deps: ['$createJsInvoker', '$createJsInvokerSignature', '$emittedFunctions'],
   $FunctionDefinition: class {
+    hasPublicSymbol = true;
     constructor(name, returnType, argumentTypes, functionIndex, thisType = null, isNonnullReturn = false, isAsync = false) {
       this.name = name;
       this.returnType = returnType;
@@ -154,6 +155,7 @@ var LibraryEmbind = {
     }
   },
   $ClassDefinition: class {
+    hasPublicSymbol = true;
     constructor(typeId, name, base = null) {
       this.typeId = typeId;
       this.name = name;
@@ -265,6 +267,7 @@ var LibraryEmbind = {
     }
   },
   $ConstantDefinition: class {
+    hasPublicSymbol = true;
     constructor(type, name) {
       this.type = type;
       this.name = name;
@@ -275,6 +278,7 @@ var LibraryEmbind = {
     }
   },
   $EnumDefinition: class {
+    hasPublicSymbol = true;
     constructor(typeId, name) {
       this.typeId = typeId;
       this.name = name;
@@ -363,8 +367,8 @@ var LibraryEmbind = {
         ['unsigned long', ['bigint']],
 #endif
 #if WASM_BIGINT
-        ['int64_t', ['bigint']],
-        ['uint64_t', ['bigint']],
+        ['long long', ['bigint']],
+        ['unsigned long long', ['bigint']],
 #endif
         ['void', ['void']],
         ['std::string', [jsString, 'string']],
@@ -418,6 +422,8 @@ var LibraryEmbind = {
           '  delete(): void;\n',
           '  deleteLater(): this;\n',
           '  isDeleted(): boolean;\n',
+          '  // @ts-ignore - If targeting lower than ESNext, this symbol might not exist.\n',
+          '  [Symbol.dispose](): void;\n',
           '  clone(): this;\n',
           '}\n',
         );
@@ -440,7 +446,7 @@ var LibraryEmbind = {
       if (this.usedEmbindString) {
         out.unshift('type EmbindString = ArrayBuffer|Uint8Array|Uint8ClampedArray|Int8Array|string;\n');
       }
-      console.log(out.join(''));
+      return out.join('');
     }
   },
 
@@ -451,14 +457,21 @@ var LibraryEmbind = {
 
     print() {
       const out = ['{\n'];
+      const publicSymbols = [];
       for (const def of this.definitions) {
+        if (def.hasPublicSymbol) {
+          publicSymbols.push(def.name);
+        }
         if (!def.printJs) {
           continue;
         }
         def.printJs(out);
       }
-      out.push('}')
-      console.log(out.join(''));
+      out.push('}\n');
+      return JSON.stringify({
+        'invokers': out.join(''),
+        publicSymbols,
+      });
     }
   },
 
@@ -468,17 +481,17 @@ var LibraryEmbind = {
   },
   $registerPrimitiveType__deps: ['$registerType', '$PrimitiveType'],
   $registerPrimitiveType: (id, name, destructorType) => {
-    name = readLatin1String(name);
+    name = AsciiToString(name);
     registerType(id, new PrimitiveType(id, name, destructorType));
   },
   $registerIntegerType__deps: ['$registerType', '$IntegerType'],
   $registerIntegerType: (id) => {
     registerType(id, new IntegerType(id));
   },
-  $createFunctionDefinition__deps: ['$FunctionDefinition', '$heap32VectorToArray', '$readLatin1String', '$Argument', '$whenDependentTypesAreResolved', '$getFunctionName', '$getFunctionArgsName', '$PointerDefinition', '$ClassDefinition'],
+  $createFunctionDefinition__deps: ['$FunctionDefinition', '$heap32VectorToArray', '$AsciiToString', '$Argument', '$whenDependentTypesAreResolved', '$getFunctionName', '$getFunctionArgsName', '$PointerDefinition', '$ClassDefinition'],
   $createFunctionDefinition: (name, argCount, rawArgTypesAddr, functionIndex, hasThis, isNonnullReturn, isAsync, cb) => {
     const argTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
-    name = typeof name === 'string' ? name : readLatin1String(name);
+    name = typeof name === 'string' ? name : AsciiToString(name);
 
     whenDependentTypesAreResolved([], argTypes, function (argTypes) {
       const argsName = getFunctionArgsName(name);
@@ -543,9 +556,9 @@ var LibraryEmbind = {
   _embind_register_emval: (rawType) => {
     registerType(rawType, new PrimitiveType(rawType, 'emscripten::val', 'none'));
   },
-  _embind_register_user_type__deps: ['$registerType', '$readLatin1String', '$UserType'],
+  _embind_register_user_type__deps: ['$registerType', '$AsciiToString', '$UserType'],
   _embind_register_user_type: (rawType, name) => {
-    name = readLatin1String(name);
+    name = AsciiToString(name);
     registerType(rawType, new UserType(rawType, name));
   },
   _embind_register_optional__deps: ['$OptionalType'],
@@ -564,7 +577,7 @@ var LibraryEmbind = {
       moduleDefinitions.push(funcDef);
     });
   },
-  _embind_register_class__deps: ['$readLatin1String', '$ClassDefinition', '$whenDependentTypesAreResolved', '$moduleDefinitions', '$PointerDefinition'],
+  _embind_register_class__deps: ['$AsciiToString', '$ClassDefinition', '$whenDependentTypesAreResolved', '$moduleDefinitions', '$PointerDefinition'],
   _embind_register_class: function(rawType,
                                   rawPointerType,
                                   rawConstPointerType,
@@ -578,7 +591,7 @@ var LibraryEmbind = {
                                   name,
                                   destructorSignature,
                                   rawDestructor) {
-    name = readLatin1String(name);
+    name = AsciiToString(name);
     whenDependentTypesAreResolved(
       [rawType, rawPointerType, rawConstPointerType],
       baseClassRawType ? [baseClassRawType] : [],
@@ -628,7 +641,7 @@ var LibraryEmbind = {
     });
   },
   _embind_register_class_property__deps: [
-    '$readLatin1String', '$whenDependentTypesAreResolved', '$ClassProperty'],
+    '$AsciiToString', '$whenDependentTypesAreResolved', '$ClassProperty'],
   _embind_register_class_property: function(classType,
                                             fieldName,
                                             getterReturnType,
@@ -639,7 +652,7 @@ var LibraryEmbind = {
                                             setterSignature,
                                             setter,
                                             setterContext) {
-    fieldName = readLatin1String(fieldName);
+    fieldName = AsciiToString(fieldName);
     const readonly = setter === 0;
     if (!(readonly || getterReturnType === setterArgumentType)) {
       throw new error('Mismatched getter and setter types are not supported.');
@@ -674,7 +687,7 @@ var LibraryEmbind = {
     });
   },
   _embind_register_class_class_property__deps: [
-    '$readLatin1String', '$whenDependentTypesAreResolved', '$ClassProperty'],
+    '$AsciiToString', '$whenDependentTypesAreResolved', '$ClassProperty'],
   _embind_register_class_class_property: (rawClassType,
                                           fieldName,
                                           rawFieldType,
@@ -683,7 +696,7 @@ var LibraryEmbind = {
                                           getter,
                                           setterSignature,
                                           setter) => {
-    fieldName = readLatin1String(fieldName);
+    fieldName = AsciiToString(fieldName);
     whenDependentTypesAreResolved([], [rawClassType], function(classType) {
       classType = classType[0];
       whenDependentTypesAreResolved([], [rawFieldType], function(types) {
@@ -696,22 +709,22 @@ var LibraryEmbind = {
   },
   // Stub function. This is called a when extending an object and not needed for TS generation.
   _embind_create_inheriting_constructor: (constructorName, wrapperType, properties) => {},
-  _embind_register_enum__deps: ['$readLatin1String', '$EnumDefinition', '$moduleDefinitions'],
+  _embind_register_enum__deps: ['$AsciiToString', '$EnumDefinition', '$moduleDefinitions'],
   _embind_register_enum: function(rawType, name, size, isSigned) {
-    name = readLatin1String(name);
+    name = AsciiToString(name);
     const enumDef = new EnumDefinition(rawType, name);
     registerType(rawType, enumDef);
     moduleDefinitions.push(enumDef);
   },
-  _embind_register_enum_value__deps: ['$readLatin1String', '$requireRegisteredType'],
+  _embind_register_enum_value__deps: ['$AsciiToString', '$requireRegisteredType'],
   _embind_register_enum_value: function(rawEnumType, name, enumValue) {
-    name = readLatin1String(name);
+    name = AsciiToString(name);
     const enumDef = requireRegisteredType(rawEnumType, name);
     enumDef.items.push([name, enumValue]);
   },
-  _embind_register_constant__deps: ['$readLatin1String', '$ConstantDefinition', '$whenDependentTypesAreResolved', '$moduleDefinitions'],
+  _embind_register_constant__deps: ['$AsciiToString', '$ConstantDefinition', '$whenDependentTypesAreResolved', '$moduleDefinitions'],
   _embind_register_constant: function(name, typeId, value) {
-    name = readLatin1String(name);
+    name = AsciiToString(name);
     whenDependentTypesAreResolved([], [typeId], function(types) {
       const def = new ConstantDefinition(types[0], name);
       moduleDefinitions.push(def);
@@ -719,7 +732,7 @@ var LibraryEmbind = {
     });
   },
   _embind_register_value_array__deps: [
-    '$readLatin1String', '$ValueArrayDefinition', '$tupleRegistrations'],
+    '$AsciiToString', '$ValueArrayDefinition', '$tupleRegistrations'],
   _embind_register_value_array: function(
     rawType,
     name,
@@ -728,7 +741,7 @@ var LibraryEmbind = {
     destructorSignature,
     rawDestructor
   ) {
-    name = readLatin1String(name);
+    name = AsciiToString(name);
     const valueArray = new ValueArrayDefinition(rawType, name);
     tupleRegistrations[rawType] = valueArray;
   },
@@ -761,7 +774,7 @@ var LibraryEmbind = {
       return [valueArray];
     });
   },
-  _embind_register_value_object__deps: ['$readLatin1String', '$ValueObjectDefinition', '$structRegistrations'],
+  _embind_register_value_object__deps: ['$AsciiToString', '$ValueObjectDefinition', '$structRegistrations'],
   _embind_register_value_object: function(
     rawType,
     name,
@@ -770,12 +783,12 @@ var LibraryEmbind = {
     destructorSignature,
     rawDestructor
   ) {
-    name = readLatin1String(name);
+    name = AsciiToString(name);
     const valueObject = new ValueObjectDefinition(rawType, name);
     structRegistrations[rawType] = valueObject;
   },
   _embind_register_value_object_field__deps: [
-    '$readLatin1String', '$structRegistrations'],
+    '$AsciiToString', '$structRegistrations'],
   _embind_register_value_object_field: function(
     structType,
     fieldName,
@@ -794,7 +807,7 @@ var LibraryEmbind = {
     }
 
     valueObject.fieldTypeIds.push(getterReturnType);
-    valueObject.fieldNames.push(readLatin1String(fieldName));
+    valueObject.fieldNames.push(AsciiToString(fieldName));
   },
   _embind_finalize_value_object__deps: ['$moduleDefinitions', '$whenDependentTypesAreResolved', '$structRegistrations'],
   _embind_finalize_value_object: function(structType) {
@@ -830,27 +843,27 @@ var LibraryEmbind = {
     });
   },
 
+  $emitOutput__deps: ['$awaitingDependencies', '$throwBindingError', '$getTypeName', '$moduleDefinitions',
 #if EMBIND_AOT
-  $embindEmitAotJs__deps: ['$awaitingDependencies', '$throwBindingError', '$getTypeName', '$moduleDefinitions', '$JsPrinter'],
-  $embindEmitAotJs__postset: () => { addAtPostCtor('embindEmitAotJs()'); },
-  $embindEmitAotJs: () => {
-    for (const typeId in awaitingDependencies) {
-      throwBindingError(`Missing binding for type: '${getTypeName(typeId)}' typeId: ${typeId}`);
-    }
-    const printer = new JsPrinter(moduleDefinitions);
-    printer.print();
-  },
-#else // EMBIND_AOT
-  $embindEmitTypes__deps: ['$awaitingDependencies', '$throwBindingError', '$getTypeName', '$moduleDefinitions', '$TsPrinter'],
-  $embindEmitTypes__postset: () => { addAtPostCtor('embindEmitTypes()'); },
-  $embindEmitTypes: () => {
-    for (const typeId in awaitingDependencies) {
-      throwBindingError(`Missing binding for type: '${getTypeName(typeId)}' typeId: ${typeId}`);
-    }
-    const printer = new TsPrinter(moduleDefinitions);
-    printer.print();
-  },
+    '$JsPrinter',
+#else
+    '$TsPrinter',
 #endif
+  ],
+  $emitOutput__postset: () => { addAtPostCtor('emitOutput()'); },
+  $emitOutput: () => {
+    for (const typeId in awaitingDependencies) {
+      throwBindingError(`Missing binding for type: '${getTypeName(typeId)}' typeId: ${typeId}`);
+    }
+#if EMBIND_AOT
+    const printer = new JsPrinter(moduleDefinitions);
+#else
+    const printer = new TsPrinter(moduleDefinitions);
+#endif
+    const output = printer.print();
+    var fs = require('fs');
+    fs.writeFileSync(process.argv[2], output + '\n');
+  },
 
   // Stub functions used by eval, but not needed for TS generation:
   $makeLegalFunctionName: () => { throw new Error('stub function should not be called'); },
@@ -861,10 +874,6 @@ var LibraryEmbind = {
   $PureVirtualError: () => { throw new Error('stub function should not be called'); },
 };
 
-#if EMBIND_AOT
-extraLibraryFuncs.push('$embindEmitAotJs');
-#else
-extraLibraryFuncs.push('$embindEmitTypes');
-#endif
+extraLibraryFuncs.push('$emitOutput');
 
 addToLibrary(LibraryEmbind);
