@@ -4,14 +4,12 @@
  * SPDX-License-Identifier: MIT
  */
 
-addToLibrary({
+var LibraryFS = {
   $FS__deps: ['$randomFill', '$PATH', '$PATH_FS', '$TTY', '$MEMFS',
     '$FS_createPreloadedFile',
     '$FS_modeStringToFlags',
     '$FS_getMode',
     '$intArrayFromString',
-    '$stringToUTF8Array',
-    '$lengthBytesUTF8',
 #if LibraryManager.has('libidbfs.js')
     '$IDBFS',
 #endif
@@ -38,9 +36,7 @@ addToLibrary({
     addAtExit('FS.quit();');
     return `
 FS.createPreloadedFile = FS_createPreloadedFile;
-FS.staticInit();
-// Set module methods based on EXPORTED_RUNTIME_METHODS
-{{{ EXPORTED_RUNTIME_METHODS.filter((func) => func.startsWith('FS_')).map((func) => "Module['" + func + "'] = FS." + func.slice(3) + ";\n").reduce((str, func) => str + func, '') }}}`;
+FS.staticInit();`;
   },
   $FS: {
     root: null,
@@ -1349,28 +1345,24 @@ FS.staticInit();
       if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
         throw new Error(`Invalid encoding type "${opts.encoding}"`);
       }
-      var ret;
       var stream = FS.open(path, opts.flags);
       var stat = FS.stat(path);
       var length = stat.size;
       var buf = new Uint8Array(length);
       FS.read(stream, buf, 0, length, 0);
       if (opts.encoding === 'utf8') {
-        ret = UTF8ArrayToString(buf);
-      } else if (opts.encoding === 'binary') {
-        ret = buf;
+        buf = UTF8ArrayToString(buf);
       }
       FS.close(stream);
-      return ret;
+      return buf;
     },
     writeFile(path, data, opts = {}) {
       opts.flags = opts.flags || {{{ cDefs.O_TRUNC | cDefs.O_CREAT | cDefs.O_WRONLY }}};
       var stream = FS.open(path, opts.flags, opts.mode);
       if (typeof data == 'string') {
-        var buf = new Uint8Array(lengthBytesUTF8(data)+1);
-        var actualNumBytes = stringToUTF8Array(data, buf, 0, buf.length);
-        FS.write(stream, buf, 0, actualNumBytes, undefined, opts.canOwn);
-      } else if (ArrayBuffer.isView(data)) {
+        data = new Uint8Array(intArrayFromString(data, true));
+      }
+      if (ArrayBuffer.isView(data)) {
         FS.write(stream, data, 0, data.byteLength, undefined, opts.canOwn);
       } else {
         throw new Error('Unsupported data type');
@@ -1917,22 +1909,23 @@ FS.staticInit();
 #endif
   },
 
-  $FS_createDataFile__deps: ['$FS'],
-  $FS_createDataFile: (parent, name, fileData, canRead, canWrite, canOwn) => {
-    FS.createDataFile(parent, name, fileData, canRead, canWrite, canOwn);
-  },
-
-  $FS_unlink__deps: ['$FS'],
-  $FS_unlink: (path) => FS.unlink(path),
-
   $FS_mkdirTree__docs: `
   /**
    * @param {number=} mode Optionally, the mode to create in. Uses mkdir's
    *                       default if not set.
    */`,
-  $FS_mkdirTree__deps: ['$FS'],
-  $FS_mkdirTree: (path, mode) => FS.mkdirTree(path, mode),
+   $FS_mkdirTree__deps: ['$FS'],
+   $FS_mkdirTree: (path, mode) => FS.mkdirTree(path, mode),
+};
 
-  $FS_createLazyFile__deps: ['$FS'],
-  $FS_createLazyFile: 'FS.createLazyFile',
-});
+// Add library aliases for all the FS.<symbol> as FS_<symbol>.
+for (let key in LibraryFS.$FS) {
+  const alias = `$FS_${key}`;
+  // Skip defining the alias if it already exists or if it's not an API function.
+  if (LibraryFS[alias] || key[0] !== key[0].toLowerCase()) {
+    continue;
+  }
+  LibraryFS[alias] = `(...args) => FS.${key}(...args)`;
+  LibraryFS[`${alias}__deps`] = ['$FS'];
+}
+addToLibrary(LibraryFS);
