@@ -53,7 +53,7 @@ var LibraryGLFW = {
       this.title = title;
       this.monitor = monitor;
       this.share = share;
-      this.attributes = Object.assign({}, GLFW.hints);
+      this.attributes = {...GLFW.hints};
       this.inputModes = {
         0x00033001:0x00034001, // GLFW_CURSOR (GLFW_CURSOR_NORMAL)
         0x00033002:0, // GLFW_STICKY_KEYS
@@ -84,6 +84,7 @@ var LibraryGLFW = {
     },
 
   $GLFW__deps: ['emscripten_get_now', '$GL', '$Browser', '$GLFW_Window',
+    'malloc', 'free',
     '$MainLoop',
     '$stringToNewUTF8',
     'emscripten_set_window_title',
@@ -836,7 +837,7 @@ var LibraryGLFW = {
       event.preventDefault();
 
 #if FILESYSTEM
-      var filenames = _malloc(event.dataTransfer.files.length*4);
+      var filenames = _malloc(event.dataTransfer.files.length * {{{ POINTER_SIZE }}});
       var filenamesArray = [];
       var count = event.dataTransfer.files.length;
 
@@ -858,7 +859,7 @@ var LibraryGLFW = {
           var data = e.target.result;
           FS.writeFile(path, new Uint8Array(data));
           if (++written === count) {
-            {{{ makeDynCall('vpii', 'GLFW.active.dropFunc') }}}(GLFW.active.id, count, filenames);
+            {{{ makeDynCall('vpip', 'GLFW.active.dropFunc') }}}(GLFW.active.id, count, filenames);
 
             for (var i = 0; i < filenamesArray.length; ++i) {
               _free(filenamesArray[i]);
@@ -870,7 +871,7 @@ var LibraryGLFW = {
 
         var filename = stringToNewUTF8(path);
         filenamesArray.push(filename);
-        {{{ makeSetValue('filenames + i*4', '0', 'filename', POINTER_TYPE) }}};
+        {{{ makeSetValue('filenames', `i*${POINTER_SIZE}` , 'filename', '*') }}};
       }
 
       for (var i = 0; i < count; ++i) {
@@ -1064,7 +1065,7 @@ var LibraryGLFW = {
     },
 
     defaultWindowHints: () => {
-      GLFW.hints = Object.assign({}, GLFW.defaultHints);
+      GLFW.hints = {...GLFW.defaultHints};
     },
 
     createWindow: (width, height, title, monitor, share) => {
@@ -1141,12 +1142,14 @@ var LibraryGLFW = {
 #endif
 
       GLFW.windows[win.id - 1] = null;
-      if (GLFW.active.id == win.id)
+      if (GLFW.active.id == win.id) {
         GLFW.active = null;
+      }
 
       // Destroy context when no alive windows
-      for (var i = 0; i < GLFW.windows.length; i++)
-        if (GLFW.windows[i] !== null) return;
+      for (win of GLFW.windows) {
+        if (win !== null) return;
+      }
 
       delete Module['ctx'];
     },
@@ -1377,7 +1380,6 @@ var LibraryGLFW = {
 /*******************************************************************************
  * GLFW FUNCTIONS
  ******************************************************************************/
-  glfwInit__deps: ['malloc', 'free'],
   glfwInit: () => {
     if (GLFW.windows) return 1; // GL_TRUE
 
