@@ -16,6 +16,7 @@ import json
 import subprocess
 import logging
 import pprint
+import re
 import shutil
 import sys
 import textwrap
@@ -455,6 +456,13 @@ def get_metadata(infile, outfile, modify_wasm, args):
   return metadata
 
 
+def is_valid_js_identifier(ident):
+  # See https://developer.mozilla.org/en-US/docs/Glossary/Identifier
+  if ident[0].isdigit():
+    return False
+  return re.fullmatch(r'[0-9a-zA-Z_\$]+', ident)
+
+
 def finalize_wasm(infile, outfile, js_syms):
   building.save_intermediate(infile, 'base.wasm')
   args = []
@@ -563,7 +571,7 @@ def finalize_wasm(infile, outfile, js_syms):
   # not known auto-generated system functions.
   unexpected_exports = [e for e in metadata.all_exports if shared.is_user_export(e)]
   for n in unexpected_exports:
-    if not n.isidentifier():
+    if not is_valid_js_identifier(n):
       exit_with_error(f'invalid export name: {n}')
   unexpected_exports = [asmjs_mangle(e) for e in unexpected_exports]
   unexpected_exports = [e for e in unexpected_exports if e not in expected_exports]
@@ -803,6 +811,8 @@ def add_standard_wasm_imports(send_items_map):
       'store_val_i64',
       'store_val_f32',
       'store_val_f64',
+      'memory_grow_pre',
+      'memory_grow_post',
     ]
 
   if settings.SPLIT_MODULE and settings.ASYNCIFY == 2:
@@ -895,78 +905,39 @@ def create_reexports(metadata):
   return exports
 
 
-def can_use_await():
-  return settings.MODULARIZE
+def install_debug_wrapper(sym):
+  if settings.MINIMAL_RUNTIME or not settings.ASSERTIONS:
+    return False
+  # The emscripten stack functions are called very early (by writeStackCookie) before
+  # the runtime is initialized so we can't create these wrappers that check for
+  # runtimeInitialized.
+  if sym.startswith(('_asan_', 'emscripten_stack_', '_emscripten_stack_')):
+    return False
+  # Likewise `__trap` can occur before the runtime is initialized since it is used in
+  # abort.
+  # pthread_self and _emscripten_proxy_execute_task_queue are currently called in some
+  # cases after the runtime has exited.
+  # TODO: Look into removing these, and improving our robustness around thread termination.
+  return sym not in {'__trap', 'pthread_self', '_emscripten_proxy_execute_task_queue'}
 
 
-def make_export_wrappers(function_exports):
-  assert not settings.MINIMAL_RUNTIME
-
-  wrappers = []
-  decls = []
-
-  def install_wrapper(sym):
-    # The emscripten stack functions are called very early (by writeStackCookie) before
-    # the runtime is initialized so we can't create these wrappers that check for
-    # runtimeInitialized.
-    if sym.startswith(('_asan_', 'emscripten_stack_', '_emscripten_stack_')):
-      return False
-    # Likewise `__trap` can occur before the runtime is initialized since it is used in
-    # abort.
-    # pthread_self and _emscripten_proxy_execute_task_queue are currently called in some
-    # cases after the runtime has exited.
-    # TODO: Look into removing these, and improving our robustness around thread termination.
-    return sym not in {'__trap', 'pthread_self', '_emscripten_proxy_execute_task_queue'}
-
-  for name, types in function_exports.items():
-    nargs = len(types.params)
-    mangled = asmjs_mangle(name)
-    wrapper = f'var {mangled} = '
-
-    # TODO(sbc): Can we avoid exporting the dynCall_ functions on the module.
-    should_export = settings.EXPORT_KEEPALIVE and mangled in settings.EXPORTED_FUNCTIONS
-    if name.startswith('dynCall_') and settings.MODULARIZE != 'instance':
-      should_export = True
-    exported = ''
-    if settings.MODULARIZE == 'instance':
-      if should_export:
-        decls.append(f'export var {mangled};')
-      else:
-        decls.append(f'var {mangled};')
-      wrapper = f'  {mangled} = '
-    elif should_export:
-      exported = "Module['%s'] = " % mangled
-      wrapper += exported
-
-    if settings.ASSERTIONS and install_wrapper(name):
-      # With assertions enabled we create a wrapper that are calls get routed through, for
-      # the lifetime of the program.
-      wrapper += f"createExportWrapper('{name}', {nargs});"
-    elif (settings.WASM_ASYNC_COMPILATION and not can_use_await()) or settings.PTHREADS or settings.WASM_WORKERS:
-      # With WASM_ASYNC_COMPILATION wrapper will replace the global var and Module var on
-      # first use.
-      args = [f'a{i}' for i in range(nargs)]
-      args = ', '.join(args)
-      wrapper += f"({args}) => ({mangled} = {exported}wasmExports['{name}'])({args});"
-    else:
-      wrapper += f"wasmExports['{name}']"
-
-    wrappers.append(wrapper)
-
-  if settings.MODULARIZE == 'instance':
-    wrappers.insert(0, 'function assignWasmExports() {')
-    wrappers.append('}')
-    wrappers = decls + wrappers
-
-  return wrappers
+def should_export(sym):
+  return settings.EXPORT_ALL or (settings.EXPORT_KEEPALIVE and sym in settings.EXPORTED_FUNCTIONS)
 
 
 def create_receiving(function_exports, tag_exports):
-  receiving = ['// Imports from the Wasm binary.']
+  generate_dyncall_assignment = settings.DYNCALLS and '$dynCall' in settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE
+  receiving = ['\n// Imports from the Wasm binary.']
 
   if settings.WASM_ESM_INTEGRATION:
-    exports = tag_exports + list(function_exports.keys())
-    exports = [f'{f} as {asmjs_mangle(f)}' for f in exports]
+    exported_symbols = tag_exports + list(function_exports.keys())
+    exports = []
+    for sym in exported_symbols:
+      mangled = asmjs_mangle(sym)
+      if mangled != sym:
+        exports.append(f'{sym} as {mangled}')
+      else:
+        exports.append(sym)
     if not settings.IMPORTED_MEMORY:
       exports.append('memory as wasmMemory')
     if not settings.RELOCATABLE:
@@ -974,6 +945,16 @@ def create_receiving(function_exports, tag_exports):
     receiving.append('import {')
     receiving.append('  ' + ',\n  '.join(exports))
     receiving.append(f"}} from './{settings.WASM_BINARY_FILE}';")
+
+    if generate_dyncall_assignment:
+      receiving.append('\nfunction assignDynCalls() {')
+      receiving.append('  // Construct dynCalls mapping')
+      for sym in function_exports:
+        if sym.startswith('dynCall_'):
+          sig_str = sym.replace('dynCall_', '')
+          receiving.append(f"  dynCalls['{sig_str}'] = {sym};")
+      receiving.append('}')
+
     return '\n'.join(receiving) + '\n\n'
 
   # When not declaring asm exports this section is empty and we instead programmatically export
@@ -981,29 +962,53 @@ def create_receiving(function_exports, tag_exports):
   if not settings.DECLARE_ASM_MODULE_EXPORTS:
     return ''
 
-  if settings.MINIMAL_RUNTIME:
-    # Exports are assigned inside a function to variables
-    # existing in top level JS scope, i.e.
-    # var _main;
-    # function assignWasmExports(wasmExport) {
-    #   _main = wasmExports["_main"];
-    generate_dyncall_assignment = settings.DYNCALLS and '$dynCall' in settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE
-    exports = [x for x in function_exports if x != building.WASM_CALL_CTORS]
-    receiving.append('function assignWasmExports(wasmExports) {')
-    for s in exports:
-      mangled = asmjs_mangle(s)
-      dynCallAssignment = ('dynCalls["' + s.replace('dynCall_', '') + '"] = ') if generate_dyncall_assignment and mangled.startswith('dynCall_') else ''
-      should_export = settings.EXPORT_ALL or (settings.EXPORT_KEEPALIVE and mangled in settings.EXPORTED_FUNCTIONS)
+  # Exports are assigned inside a function to variables
+  # existing in top level JS scope, i.e.
+  # var _main;
+  # function assignWasmExports(wasmExport) {
+  #   _main = wasmExports["_main"];
+  exports = {name: sig for name, sig in function_exports.items() if name != building.WASM_CALL_CTORS}
+
+  if settings.ASSERTIONS:
+    # In debug builds we generate trapping functions in case
+    # folks try to call/use a reference that was taken before the
+    # wasm module is available.
+    for sym in exports:
+      mangled = asmjs_mangle(sym)
       export_assignment = ''
-      if settings.MODULARIZE and should_export:
-        export_assignment = f"Module['{mangled}'] = "
-      receiving.append(f"  {export_assignment}{dynCallAssignment}{mangled} = wasmExports['{s}'];")
-    receiving.append('}')
+      if (settings.MODULARIZE or not settings.MINIMAL_RUNTIME) and should_export(mangled) and settings.MODULARIZE != 'instance':
+        export_assignment = f" = Module['{mangled}']"
+      receiving.append(f"var {mangled}{export_assignment} = makeInvalidEarlyAccess('{mangled}');")
+  else:
+    # Declare all exports in a single var statement
     sep = ',\n  '
     mangled = [asmjs_mangle(s) for s in exports]
-    receiving.append(f'var {sep.join(mangled)};')
-  else:
-    receiving += make_export_wrappers(function_exports)
+    receiving.append(f'var {sep.join(mangled)};\n')
+
+  if settings.MODULARIZE == 'instance':
+    mangled = [asmjs_mangle(e) for e in exports]
+    esm_exports = [e for e in mangled if should_export(e)]
+    if esm_exports:
+      esm_exports = ', '.join(esm_exports)
+      receiving.append(f'export {{ {esm_exports} }};')
+
+  receiving.append('\nfunction assignWasmExports(wasmExports) {')
+  for sym, sig in exports.items():
+    mangled = asmjs_mangle(sym)
+    if generate_dyncall_assignment and sym.startswith('dynCall_'):
+      sig_str = sym.replace('dynCall_', '')
+      dynCallAssignment = f"dynCalls['{sig_str}'] = "
+    else:
+      dynCallAssignment = ''
+    export_assignment = ''
+    if (settings.MODULARIZE or not settings.MINIMAL_RUNTIME) and should_export(mangled) and settings.MODULARIZE != 'instance':
+      export_assignment = f"Module['{mangled}'] = "
+    if install_debug_wrapper(sym):
+      nargs = len(sig.params)
+      receiving.append(f"  {export_assignment}{dynCallAssignment}{mangled} = createExportWrapper('{sym}', {nargs});")
+    else:
+      receiving.append(f"  {export_assignment}{dynCallAssignment}{mangled} = wasmExports['{sym}'];")
+  receiving.append('}')
 
   return '\n'.join(receiving) + '\n'
 
@@ -1015,10 +1020,12 @@ def create_module(metadata, function_exports, global_exports, tag_exports,librar
   receiving += create_global_exports(global_exports)
   sending = create_sending(metadata, library_symbols)
 
+  module.append(receiving)
+
   if settings.WASM_ESM_INTEGRATION:
     module.append(sending)
   else:
-    receiving += create_other_export_declarations(tag_exports)
+    module.append(create_other_export_declarations(tag_exports))
 
     if settings.PTHREADS or settings.WASM_WORKERS or (settings.IMPORTED_MEMORY and settings.MODULARIZE == 'instance'):
       sending = textwrap.indent(sending, '  ').strip()
@@ -1035,7 +1042,7 @@ def create_module(metadata, function_exports, global_exports, tag_exports,librar
       if settings.MODULARIZE == 'instance':
         module.append("var wasmExports;\n")
       elif settings.WASM_ASYNC_COMPILATION:
-        if can_use_await():
+        if settings.MODULARIZE:
           # In modularize mode the generated code is within a factory function.
           # This magic string gets replaced by `await createWasm`.  It needed to allow
           # closure and acorn to process the module without seeing this as a top-level
@@ -1046,8 +1053,6 @@ def create_module(metadata, function_exports, global_exports, tag_exports,librar
       else:
         module.append("var wasmExports = createWasm();\n")
 
-  module.append(receiving)
-
   if settings.SUPPORT_LONGJMP == 'emscripten' or not settings.DISABLE_EXCEPTION_CATCHING:
     module.append(create_invoke_wrappers(metadata))
   else:
@@ -1088,6 +1093,7 @@ def create_pointer_conversion_wrappers(metadata):
     'emscripten_builtin_calloc': 'ppp',
     'wasmfs_create_node_backend': 'pp',
     'malloc': 'pp',
+    'realloc': 'ppp',
     'calloc': 'ppp',
     'webidl_malloc': 'pp',
     'memalign': 'ppp',
@@ -1158,6 +1164,7 @@ def create_pointer_conversion_wrappers(metadata):
     'emscripten_proxy_finish': '_p',
     'emscripten_proxy_execute_queue': '_p',
     '_emval_coro_resume': '_pp',
+    '_emval_coro_reject': '_pp',
     'emscripten_main_runtime_thread_id': 'p',
     '_emscripten_set_offscreencanvas_size_on_thread': '_pp__',
     'fileno': '_p',
